package aventi.epark.navigation;

import android.util.Log;

import org.altbeacon.beacon.Beacon;
import org.altbeacon.beacon.Identifier;
import org.apache.commons.math3.fitting.leastsquares.LeastSquaresOptimizer;
import org.apache.commons.math3.fitting.leastsquares.LeastSquaresOptimizer.Optimum;
import org.apache.commons.math3.fitting.leastsquares.LevenbergMarquardtOptimizer;

import java.util.Collection;
import java.util.List;

import aventi.epark.navigation.trilateration.NonLinearLeastSquaresSolver;
import aventi.epark.navigation.trilateration.TrilaterationFunction;
import aventi.epark.utilities.BeaconDistance;

/**
 * Created by espeland on 01.03.18.
 */

public class Multilateration extends BeaconDistance {

    /**
     * Pre-defined coordinates of Beacons {x,y}
     * Coordinate-system is placed in the TOP_LEFT_CORNER,
     * and beacon ID should be put ID. 1-8 in array [0]-[7]*/
    private double[][] XY_FOUR_BEACONS = new double[][] {{0,-13.8},{2.5,-13.8},{0,0},{2.5,0}};
    private double[][] XY_SIX_BEACONS = new double[][] {{0,-13.8},{2.5,-13.8},{0,0},{2.5,0},{0,-4.6},{2.5,-4.6}};
    private double[][] XY_EIGHT_BEACONS = new double[][] {{0,-13.8},{2.5,-13.8},{0,-9.2},{2.5,-9.2},{0,-4.6},{2.5,-4.6},{0,0},{2.4,0}};

    public String str;


    private Collection<Beacon> beacons;

    public Multilateration(Collection<Beacon> beacons) {
        this.beacons = beacons;
    }

    public double[] getDistances(Collection<Beacon> beacons) {
        double[] distances = new double[4];

        for (Beacon beacon : beacons) {
            Log.i("Beacon Services", "Checking if UID matches Eddystone Standard");
            if (beacon.getServiceUuid() == 0xfeaa && beacon.getBeaconTypeCode() == 0x00) {
                int beaconID = identifierToInt(beacon.getId2());
                if (beaconID > 0 && beaconID < 9) {
                    distances[beaconID-1] = calculateDistance(beacon);
                    if (beaconID == 1) {
                        str = String.valueOf(calculateDistance(beacon));
                    }
                }
            }
        }
        return distances;
    }


    public double[] getCurrentLocation() {
        NonLinearLeastSquaresSolver solver = new NonLinearLeastSquaresSolver(new TrilaterationFunction(XY_FOUR_BEACONS, getDistances(beacons)), new LevenbergMarquardtOptimizer());
        Optimum optimum = solver.solve();

        return optimum.getPoint().toArray();
    }

    public int identifierToInt(Identifier id) {
        int sum = 0;
        String str = id.toString();
        int index = 13;
        int p = 0;
        while (index > 1 && str.charAt(index) != '0'){
            int pow = (int) Math.pow(16.0,(double) p);
            if (str.charAt(index) > '0' && str.charAt(index) <= '9'){
                sum += (((int)str.charAt(index))-48)*pow;
            }else{
                sum += (((int)str.charAt(index))-87)*pow;
            }
            index--;
            p++;
        }
        return sum;
    }
}
